#pragma once

#include <memory>
#include <vector>
#include <iostream>
#include <cstring>
#include <string>

#include "Types.hpp"
#include "MemoryBankController.hpp"
#include "mbc/MemBC.hpp"

class Debugger;

class CartRidge
{
public:
  CartRidge(Debugger& debugger);

  // delete copy constructor, can't used defaulted one, because of unique_ptr
  CartRidge(const CartRidge&) = delete;

  void readFromFile(const std::string& filename);

  GBByte readByteRom0(GBPtr src);
  GBByte readByteRomN(GBPtr src);



  void writeByteRom(GBPtr dst, GBByte data);

private:
  std::unique_ptr<MemBC> m_mbc;
  std::vector<std::unique_ptr<GBByte[]>> m_romBanks;
  std::unique_ptr<GBByte[]> m_ram;
  std::string m_gameTitle;
  Debugger& m_debugger;
  uint8_t m_mbcBPRanges;
  bool m_mbcBP;

  void setupHardware(int cartRidgeType, int romSizeByte,
                     int ramSizeByte);

  // friend declarations for output and testing
  friend std::ostream& operator<<(std::ostream& out, const CartRidge& cr);
  friend class CartRidgeTest;
};

std::ostream& operator<<(std::ostream& out, const CartRidge& cr);   