#pragma once
#include <vector>
#include <functional>
#include <string>
#include <map>

#include "Types.hpp"

class GameBoy;
class Processor;


/*******************************************************************************
 * Enumeration for output level
 */
enum OutputLevel { OL_ERROR, OL_WARNING, OL_DEBUG, OL_VERBOSE };


/*******************************************************************************
 * @brief Handles breakpoints, user commands and everything else to debug 
 *        the execution
 */
class Debugger 
{
public:
  Debugger(GameBoy& gb);
  
  /// returns the current output level
  OutputLevel getOutputLevel() const { return m_outputLevel; }

  /// checks for breakpoints, read user input, etc
  void check();
  /// should be called whenever something is written to memory
  void writeAccess(GBPtr ptr, GBByte data);
  /// should be called whenever something is read from memory
  void readAccess(GBPtr ptr, GBByte data);

  /// pause execution
  void pause(std::string reason = "");
  /// resume execution
  void resume();
  /// wether or not execution is paused
  bool isPaused() const { return m_paused; }

  // typedefs
  typedef std::vector<std::string> ArgList;
  typedef std::function<bool(ArgList)> DebugCommand;

  // user defined commands
  void addCommand(const std::string& name, DebugCommand func);

private:
  GameBoy& m_gb;
  Processor& m_cpu;

  bool m_paused;
  OutputLevel m_outputLevel;

  std::vector<GBPtr> m_pcBP;
  std::vector<GBByte> m_insBP;
  std::vector<GBPtr> m_memBP;
  std::map<std::string, DebugCommand> m_commands;

  bool checkForBreakPoint(GBByte nextIns);
  void handleUserInput();
  void addDebugCommands();
};
