#pragma once

#include "Processor.hpp"
#include "GraphicProcessor.hpp"
#include "Timer.hpp"
#include "CartRidge.hpp"
#include "HardwareRegisters.hpp"
#include "Types.hpp"
#include "../GLWindow.hpp"
#include "Debugger.hpp"

#include <memory>
#include <iostream>

class GameBoy
{
public:
  GameBoy();
  ~GameBoy() {}

  void byteCopy(GBPtr dst, GBPtr src)
  {
    writeByte(dst, readByte(src));
  }
  void wordCopy(GBPtr dst, GBPtr src)
  {
    writeWord(dst, readWord(src));
  }

  void writeByte(GBPtr dst, GBByte data);
  void writeByteImpl(GBPtr dst, GBByte data);
  void writeRegister(GBByte dst, GBByte data);
  void writeWord(GBPtr dst, GBWord data)
  {
    writeByte(dst + 1, static_cast<GBByte>(data >> 8));
    writeByte(dst, static_cast<GBByte>(data));
  }

  GBByte readByte(GBPtr ptr);
  GBByte readByteImpl(GBPtr ptr);
  GBByte readRegister(GBByte ptr);
  GBWord readWord(GBPtr ptr)
  {
    return ((readByte(ptr + 1) << 8) | readByte(ptr));
  }

  void runDevice();
  GLWindow* getWindow() { return &m_window; }
  HardwareRegisters* getRegisters() { return &m_registers; }
  Debugger* getDebugger() { return &m_debugger; }
  Processor& getCPU() { return m_CPU; }

private:
  // Game Boy internals
  GBByte m_interruptEnableRegister;
  Processor m_CPU;
  Debugger m_debugger;
  GraphicProcessor m_GPU;
  Timer m_timer;
  CartRidge m_cartRidge;
  HardwareRegisters m_registers;
  std::unique_ptr<uint8_t[]> m_workRam;
  std::unique_ptr<GBByte[]> m_highRam;

  // Window handling
  GLWindow m_window;

  friend std::ostream& operator<<(std::ostream& out, const GameBoy& gb);
};

std::ostream& operator<<(std::ostream& out, const GameBoy& gb);
