#pragma once

#include <memory>
#include <vector>
#include "Types.hpp"

class GameBoy;

// VRAM
struct Tile
{
  GBByte data[16]; // 8x8 Pixels, 2 Bytes per row
};

struct Background
{
  GBByte data[32*32]; // 32x32 Tiles
};

struct VRAM
{
  Tile tiles[384];
  Background backgroundMap[2];
};
static_assert(sizeof(VRAM) == 8192, "Invalid VRAM size");

// Sprite attribute table
struct SpriteAttribute
{
  GBByte yPos, xPos;
  GBByte tileNum;
  GBByte flags;
};

class GraphicProcessor
{
public:
  GraphicProcessor(GameBoy* gb);
  ~GraphicProcessor();

  GBByte* getVRam() const
  {
    return reinterpret_cast<GBByte*>(m_vRam.get());
  }

  GBByte* getSpriteAttributes() const
  {
    return reinterpret_cast<GBByte*>(m_sprites.get());
  }

  void cycle();

private:
  // Internal functions
  void draw();
  void drawLine();
  void buildSpriteTable();

  // Game Boy resources
  GameBoy* m_gb;
  std::unique_ptr<VRAM> m_vRam;
  std::unique_ptr<SpriteAttribute[]> m_sprites;

  // Rendering
  uint8_t m_cycleCount;
  std::vector<uint8_t> m_spritePriority;
  std::unique_ptr<uint8_t[]> m_screen;
  std::unique_ptr<uint8_t[]> m_virtualScreen;
};
