#pragma once

#include "Types.hpp"

struct HardwareRegisters
{
  GBByte JOYP;    // FF00 Joypad (R/W)
  GBByte SB;      // FF01 Serial Transfer Data (R/W)
  GBByte SC;      // FF02 Serial Transfer Control (R/W)
  GBByte unused0;
  GBByte DIV;     // FF04 Divider Register (R/W)
  GBByte TIMA;    // FF05 Timer Counter (R/W)
  GBByte TMA;     // FF06 Timer Modulo (R/W)
  GBByte TAC;     // FF07 Timer Control (R/W)
  GBByte unused1[7];
  GBByte IF;      // FF0F Interrupt Flag (R/W)
  GBByte NR10;    // FF10 Channel 1 Sweep register (R/W)
  GBByte NR11;    // FF11 Channel 1 Sound length/Wave Pattern Duty (R/W)
  GBByte NR12;    // FF12 Channel 1 Volume Envelope (R/W)
  GBByte NR13;    // FF13 Channel 1 Frequency lo (W)
  GBByte NR14;    // FF14 Channel 1 Frequency hi (R/W)
  GBByte unused2;
  GBByte NR21;    // FF16 Channel 2 Sound length/Wave Pattern Duty (R/W)
  GBByte NR22;    // FF17 Channel 2 Volume Envelope (R/W)
  GBByte NR23;    // FF18 Channel 2 Frequency lo (W)
  GBByte NR24;    // FF19 Channel 2 Frequency hi (R/W)
  GBByte NR30;    // FF1A Channel 3 Sound on/off (R/W)
  GBByte NR31;    // FF1B Channel 3 Sound Length (R/W)
  GBByte NR32;    // FF1C Channel 3 Select output level (R/W)
  GBByte NR33;    // FF1D Channel 3 Frequency lo (W)
  GBByte NR34;    // FF1E Channel 3 Frequency hi (R/W)
  GBByte unused3;
  GBByte NR41;    // FF20 Channel 4 Sound Length (R/W)
  GBByte NR42;    // FF21 Channel 4 Volume Envelope (R/W)
  GBByte NR43;    // FF22 Channel 4 Polynomial Counter (R/W)
  GBByte NR44;    // FF23 Channel 4 Counter/consecutive; Inital (R/W)
  GBByte NR50;    // FF24 Channel control / ON-OFF / Volume (R/W)
  GBByte NR51;    // FF25 Selection of Sound output terminal (R/W)
  GBByte NR52;    // FF26 Sound on/off
  GBByte unused4[9];
  GBByte WAV[16]; // FF30-FF3F Wave Pattern RAM
  GBByte LCDC;    // FF40 LCD Control (R/W)
  GBByte STAT;    // FF41 LCDC Status (R/W)
  GBByte SCY;     // FF42 Scroll Y (R/W)
  GBByte SCX;     // FF43 Scroll X (R/W)
  GBByte LY;      // FF44 LCDC Y-Coordinate (R)
  GBByte LYC;     // FF45 LY Compare (R/W)
  GBByte DMA;     // FF46 DMA Transfer and Start Address (W)
  GBByte BGP;     // FF47 BG Palette Data (R/W)
  GBByte OBP0;    // FF48 Object Palette 0 Data (R/W)
  GBByte OBP1;    // FF49 Object Palette 1 Data (R/W)
  GBByte WY;      // FF4A Window Y Position (R/W)
  GBByte WX;      // FF4B Window X Position minus 7 (R/W)
  GBByte unused5[4];
  GBByte ROMD;    // FF50 Internal Rom Disable
  GBByte unused6[47];
};
static_assert(sizeof(HardwareRegisters) == 128, "Invalid hardware register count");


// Read mask is ORed with data in register before returning
// (non readable bits are always set)
const GBByte regReadMask[128] = 
//  0    1    2    3    4    5    6    7    8    9    A    B    C    D    E    F
   {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 0
    0x80,0x3F,0x00,0xFF,0xBF,0xFF,0x3F,0x00,0xFF,0xBF,0x7F,0xFF,0x9F,0xFF,0xBF,0xFF, // 1
    0xFF,0x00,0x00,0xBF,0x00,0x00,0x70,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, // 2
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 3
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 4
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 5
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 6
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};// 7


const GBByte regWriteMask[128] = 
//  0    1    2    3    4    5    6    7    8    9    A    B    C    D    E    F
   {0xF0,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, // 0
    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, // 1
    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x80,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, // 2
    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, // 3
    0xFF,0xF8,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, // 4
    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, // 5
    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, // 6
    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};// 7
