#pragma once

#include "Types.hpp"

#include <iostream>
#include <cstring>


class MemoryBankController
{
public:
  MemoryBankController()
    : m_controlRegister(0),
      m_controlRegisterB(0),
      m_mbcVersion(-1),
      m_romNumber(0),
      m_ramNumber(0),
      m_romModeEnabled(true)
  {
    memset(m_rtc, 0, sizeof(m_rtc));
  }

  void create(int version);

  int currentRomBankNumber() const;
  int currentRamBankNumber() const;

  void writeControllA(GBByte data);
  void writeControllB(GBByte data, GBPtr ptr);
  void writeControllC(GBByte data);
  void writeControllD(GBByte data);


private:
  // Bit 0-4: lower bits of rom bank number
  // Bit 5-6: upper bits of rom bank number (if romModeEnabled)
  //          otherwise: RAM bank number
  // Bit 7: RAM Enable
  uint8_t m_controlRegister;
  // just used in MBC3:
  // Bit 0-3: RAM bank number or RTC select
  uint8_t m_controlRegisterB;
  uint8_t m_mbcVersion;
  uint16_t m_romNumber;
  uint16_t m_ramNumber;
  bool m_romModeEnabled;

  uint8_t m_rtc[5];   // not always used (real time clock)


  // friend declarations for output and testing
  friend std::ostream& operator<<(std::ostream& out,
                                  const MemoryBankController& cr);
  friend class CartRidgeTest;
};

std::ostream& operator<<(std::ostream& out, const MemoryBankController& cr);