#include "MemBC1.hpp"

#include <iostream>
using namespace std;

int MemBC1::romNumber() const
{
  // extract either 7 or 5 bits from m_reg
  return m_romMode ? m_reg & 0x7F : m_reg & 0x1F;
}

int MemBC1::ramNumber() const
{
  return m_romMode ? 0 : (m_reg & 0x60) >> 5;
}

void MemBC1::controllWrite(GBPtr ptr, GBByte data)
{
  switch((ptr & 0x6000) >> 13)
  {
    case 0:  // 0000h - 1FFFh
    {
      // set highest bit (ram enable)
      m_reg = (m_reg & 0x7F) | ((data & 0x0F) == 0x0A) << 7;
    }
    case 1:  // 2000h - 3FFFh
    {
      // set 5 lower bits
      m_reg = (m_reg & 0xE0) | (data & 0x1F);

      // correct value if lower 5 bits equals 0
      if((m_reg & 0x1F) == 0)
        m_reg++;
    }
    case 2:  // 4000h - 5FFFh
    {
      // set bit 5 and 6
      m_reg = (m_reg & 0x9F) | (data & 0x60);
    }
    case 3:  // 6000h - 7FFFh
    {
      // 0 -> rom, 1 -> ram
      m_romMode = (data == 0);
    }
  }
}