#include "MemBC3.hpp"

#include "MemBC1.hpp"

#include <iostream>
using namespace std;

int MemBC3::romNumber() const
{
  // lower 7 bits
  return (m_romReg & 0x7F);
}

int MemBC3::ramNumber() const
{
  // lower two bits
  return (m_ramReg & 0x03);
}

void MemBC3::controllWrite(GBPtr ptr, GBByte data)
{
  switch((ptr & 0x6000) >> 13)
  {
    case 0:  // 0000h - 1FFFh
    {
      // set third bit (ram enable)
      m_ramReg = (m_ramReg & 0x03) | ((data & 0x0F) == 0x0A) << 2;
    }
    case 1:  // 2000h - 3FFFh
    {
      // set 7 lower bits
      m_romReg = (data & 0x7F);

      // correct value if lower 5 bits equals 0
      if(m_romReg == 0)
        m_romReg++;
    }
    case 2:  // 4000h - 5FFFh
    {
      // set two lowest bits
      m_ramReg = (m_ramReg & 0xFC) | (data & 0x03);
      // TODO: RTC
    }
    case 3:  // 6000h - 7FFFh
    {
      // TODO: Latch Clock Data
    }
  }
}